<?php

namespace Teknisa\FAI\Modules\Tabelas\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use \Teknisa\Model\Mymenu\Entities;

class UnidadeClienteUsuario extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = 'unidade_cliente_usuario';

    protected $crudService;
    protected $parameterBag;
    protected $entityManager;
    protected $newCode;
    protected $environment;

    public function __construct(
        $crudService, $parameterBag, $entityManager, $newCode,
        \Teknisa\GER\Utilities\Environment $environment
    ) {
        parent::__construct($crudService, $parameterBag);

        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
        $this->environment = $environment;
    }

    public function salvaNovoUsuarioMyOrder(
        DTO\Request\Row $request, DTO\Response $response
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $rows = $request->getRow()[0];

            $nrOrg = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            $data = $this->environment->getCurrentTime();

            $codigoUsuario = $this->salvaUsuario(
                $rows['usuario'], $nrOrg, $operador, $data);

            $codigoAssociacao = $this->salvaAssocUsuarioAplicativo(
                $codigoUsuario, $nrOrg, $operador, $data);

            $this->salvaUnidadeClienteUsuarioComPapel(
                $codigoAssociacao, $rows['unidadeCliente'], $rows['papel'],
                $nrOrg, $operador, $data);

            $connection->commit();
            return true;
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function salvaUsuario($usuario, $nrOrg, $operador, $data){
        try {
            /** @var Entities\Usuarioapp $dadosUsuario */
            $dadosUsuario = $this->entityManager->getRepository(
                '\Teknisa\Model\Mymenu\Entities\Usuarioapp'
            )
                ->findOneBy(
                    array(
                        'nrorg' => $nrOrg,
                        'dsemailusuarioapp' => $usuario['DSEMAILUSUARIOAPP']
                    )
                );

            if ($dadosUsuario instanceof Entities\Usuarioapp) {
                $codigo = $dadosUsuario->getNrusuarioapp();
            } else {
                $codigo = $this->newCode->getCodeByTableName(
                    "USUARIOAPP", $nrOrg
                );

                $usuarioapp = new Entities\Usuarioapp();
                $usuarioapp->setNrusuarioapp($codigo);
                $usuarioapp->setDsemailusuarioapp(
                    $usuario['DSEMAILUSUARIOAPP']
                );
                $usuarioapp->setDsprimeironome($usuario['DSPRIMEIRONOME']);
                $usuarioapp->setDsultimonome($usuario['DSULTIMONOME']);
                $usuarioapp->setDstelefone($usuario['DSTELEFONE']);
                $usuarioapp->setIdativo('S');
                $usuarioapp->setNrorg($nrOrg);
                $usuarioapp->setNrorginclusao($nrOrg);
                $usuarioapp->setCdoperinclusao($operador);
                $usuarioapp->setDtinclusao($data);

                $this->entityManager->persist($usuarioapp);
                $this->entityManager->flush();
            }

            return $codigo;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvaAssocUsuarioAplicativo(
        $codigoUsuario, $nrOrg, $operador, $data
    ){
        try {
            /** @var Entities\AppUsuarioaplicativo $dadosUsuario */
            $dadosUsuario = $this->entityManager->getRepository(
                '\Teknisa\Model\Mymenu\Entities\AppUsuarioaplicativo'
            )
                ->findOneBy(
                    array(
                        'nrorg' => $nrOrg,
                        'nrusuarioapp' => $codigoUsuario
                    )
                );

            if ($dadosUsuario instanceof Entities\AppUsuarioaplicativo) {
                $codigo = $dadosUsuario->getNrusuarioaplicativo();
            } else {
                $codigo = $this->newCode->getCodeByTableName(
                    "app_usuarioaplicativo", $nrOrg
                );

                $assocUsuarioAplic = new Entities\AppUsuarioaplicativo();
                $assocUsuarioAplic->setNrusuarioaplicativo($codigo);
                $assocUsuarioAplic->setNrusuarioapp($codigoUsuario);
                $assocUsuarioAplic->setNraplicativo(3);
                $assocUsuarioAplic->setIdativo('S');
                $assocUsuarioAplic->setNrorg($nrOrg);
                $assocUsuarioAplic->setNrorginclusao($nrOrg);
                $assocUsuarioAplic->setCdoperinclusao($operador);
                $assocUsuarioAplic->setDtinclusao($data);

                $this->entityManager->persist($assocUsuarioAplic);
                $this->entityManager->flush();
            }

            return $codigo;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function salvaUnidadeClienteUsuarioComPapel(
        $codigoAssociacao, $unidadeCliente, $papeis, $nrOrg, $operador, $data
    ){
        try {
            $unidades = $unidadeCliente['CDFILIAL'];
            $clientes = $unidadeCliente['CDCLIENTE'];

            $clientesTratados = $this->trataClientes($unidades, $clientes);
            $cliente = explode("_", $clientesTratados);

            foreach ($cliente as $cli) {
                /** @var Entities\AppUsuarioclifil $usuarioCliFil */
                $usuarioCliFil = $this->entityManager->getRepository(
                    '\Teknisa\Model\Mymenu\Entities\AppUsuarioclifil'
                )
                    ->findOneBy(
                        array(
                            'nrorg' => $nrOrg,
                            'nrusuarioaplicativo' => $codigoAssociacao,
                            'cdfilial' => substr($cli, 0, 4),
                            'cdcliente' => substr($cli, 4)
                        )
                    );

                if ($usuarioCliFil instanceof Entities\AppUsuarioclifil) {
                    $codigoUsCliFil = $usuarioCliFil->getNrusuarioclifil();
                } else {
                    $codigoUsCliFil = $this->newCode->getCodeByTableName(
                        "app_usuarioclifil", $nrOrg
                    );

                    $assocUsuarioCliFil = new Entities\AppUsuarioclifil();
                    $assocUsuarioCliFil->setNrusuarioclifil($codigoUsCliFil);
                    $assocUsuarioCliFil->setNrusuarioaplicativo(
                        $codigoAssociacao
                    );
                    $assocUsuarioCliFil->setIdselecionado('N');
                    $assocUsuarioCliFil->setCdfilial(substr($cli, 0, 4));
                    $assocUsuarioCliFil->setCdcliente(substr($cli, 4));
                    $assocUsuarioCliFil->setIdativo('S');
                    $assocUsuarioCliFil->setNrorg($nrOrg);
                    $assocUsuarioCliFil->setNrorginclusao($nrOrg);
                    $assocUsuarioCliFil->setCdoperinclusao($operador);
                    $assocUsuarioCliFil->setDtinclusao($data);

                    $this->entityManager->persist($assocUsuarioCliFil);
                    $this->entityManager->flush();
                }

                foreach ($papeis as $papel) {
                    if ($papel <> 'N') {
                        /** @var Entities\AppPapelusuclifil $papelCliFil */
                        $papelCliFil = $this->entityManager->getRepository(
                            '\Teknisa\Model\Mymenu\Entities\AppPapelusuclifil'
                        )
                            ->findOneBy(
                                array(
                                    'nrorg' => $nrOrg,
                                    'nrusuarioclifil' => $codigoUsCliFil,
                                    'nrpapel' => $papel
                                )
                            );

                        if (!($papelCliFil instanceof Entities\AppPapelusuclifil)) {
                            $codigoPapel = $this->newCode->getCodeByTableName(
                                "app_papelusuclifil", $nrOrg
                            );

                            $dadosPapel = new Entities\AppPapelusuclifil();
                            $dadosPapel->setNrusuarioclifil($codigoUsCliFil);
                            $dadosPapel->setNrpapelusuclifil($codigoPapel);
                            $dadosPapel->setNrpapel($papel);
                            $dadosPapel->setIdativo('S');
                            $dadosPapel->setNrorg($nrOrg);
                            $dadosPapel->setNrorginclusao($nrOrg);
                            $dadosPapel->setCdoperinclusao($operador);
                            $dadosPapel->setDtinclusao($data);

                            $this->entityManager->persist($dadosPapel);
                            $this->entityManager->flush();
                        }
                    }
                }
            }

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function trataClientes($unidades, $clientes){
        if ($clientes === 'T') {
            if ($unidades === 'T') {
                $dataSourceName = "filial_operador_grupfili";
                $conditions = array(
                    array(
                        "columnName" => "CDOPERADOR",
                        "operator"   => '=',
                        "value"      => $this->environment->getCdOperador()
                    ),
                    array(
                        "columnName" => "CDGRUPFILI",
                        "operator"   => '=',
                        "value"      => "T"
                    )
                );
                $retornaUnidades = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
                if (count($retornaUnidades->getRows()) > 0) {
                    $unidades = implode('_', array_column($retornaUnidades->getRows(), "CDFILIAL"));
                }
            }

            $dataSourceName = "cliente_clienfilial";
            $conditions = array(
                array(
                    "columnName" => "NRORG",
                    "operator"   => '=',
                    "value"      => $this->environment->getNrOrg()
                ),
                array(
                    "columnName" => "CDFILIAL",
                    "operator"   => '=',
                    "value"      => $unidades
                )
            );

            $retornaClientes = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
            if (count($retornaClientes->getRows()) > 0) {
                $clientes = implode('_', array_column($retornaClientes->getRows(), "FILCLI"));
            } else {
                $clientes = "";
            }
        }

        return $clientes;
    }

    public function editaUsuarioMyOrder(
        DTO\Request\Row $request, DTO\Response $response
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $row = $request->getRow();

            $nrOrg = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            $data = $this->environment->getCurrentTime();

            $this->trataAprovacao(
                $nrOrg, $row[0]['NRUSUARIOCLIFIL'], $row[1]['APROVA'],
                $operador, $data);

            $this->trataSolicitacao(
                $nrOrg, $row[0]['NRUSUARIOCLIFIL'], $row[1]['SOLICITA'],
                $operador, $data);

            $connection->commit();
            return true;
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function trataAprovacao(
        $nrOrg, $usuarioCliFil, $idAprova, $operador, $data
    ){
        /** @var Entities\AppPapelusuclifil $aprovacao */
        $aprovacao = $this->entityManager->getRepository(
            '\Teknisa\Model\Mymenu\Entities\AppPapelusuclifil'
        )
            ->findOneBy(
                array(
                    'nrorg' => $nrOrg,
                    'nrusuarioclifil' => $usuarioCliFil,
                    'nrpapel' => 2
                )
            );

        if ($aprovacao instanceof Entities\AppPapelusuclifil and $idAprova === 'N') {
            $this->entityManager->remove($aprovacao);
            $this->entityManager->flush();
        } else if (!($aprovacao instanceof Entities\AppPapelusuclifil) and $idAprova == 2) {
            $codigoPapel = $this->newCode->getCodeByTableName(
                "app_papelusuclifil", $nrOrg
            );

            $dadosPapel = new Entities\AppPapelusuclifil();
            $dadosPapel->setNrusuarioclifil($usuarioCliFil);
            $dadosPapel->setNrpapelusuclifil($codigoPapel);
            $dadosPapel->setNrpapel($idAprova);
            $dadosPapel->setIdativo('S');
            $dadosPapel->setNrorg($nrOrg);
            $dadosPapel->setNrorginclusao($nrOrg);
            $dadosPapel->setCdoperinclusao($operador);
            $dadosPapel->setDtinclusao($data);

            $this->entityManager->persist($dadosPapel);
            $this->entityManager->flush();
        }
    }

    public function trataSolicitacao(
        $nrOrg, $usuarioCliFil, $idSolicita, $operador, $data
    ){
        /** @var Entities\AppPapelusuclifil $solicitacao */
        $solicitacao = $this->entityManager->getRepository(
            '\Teknisa\Model\Mymenu\Entities\AppPapelusuclifil'
        )
            ->findOneBy(
                array(
                    'nrorg' => $nrOrg,
                    'nrusuarioclifil' => $usuarioCliFil,
                    'nrpapel' => 1
                )
            );

        if ($solicitacao instanceof Entities\AppPapelusuclifil and $idSolicita === 'N') {
            $this->entityManager->remove($solicitacao);
            $this->entityManager->flush();
        } else if (!($solicitacao instanceof Entities\AppPapelusuclifil) and $idSolicita == 1) {
            $codigoPapel = $this->newCode->getCodeByTableName(
                "app_papelusuclifil", $nrOrg
            );

            $dadosPapel = new Entities\AppPapelusuclifil();
            $dadosPapel->setNrusuarioclifil($usuarioCliFil);
            $dadosPapel->setNrpapelusuclifil($codigoPapel);
            $dadosPapel->setNrpapel($idSolicita);
            $dadosPapel->setIdativo('S');
            $dadosPapel->setNrorg($nrOrg);
            $dadosPapel->setNrorginclusao($nrOrg);
            $dadosPapel->setCdoperinclusao($operador);
            $dadosPapel->setDtinclusao($data);

            $this->entityManager->persist($dadosPapel);
            $this->entityManager->flush();
        }
    }

    public function incluiCliFilMyOrder(
        DTO\Request\Row $request, DTO\Response $response
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $rows = $request->getRow()[0];

            $nrOrg = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            $data = $this->environment->getCurrentTime();

            /** @var Entities\AppUsuarioaplicativo $usuarioAssocApp */
            $usuarioAssocApp = $this->entityManager->getRepository(
                '\Teknisa\Model\Mymenu\Entities\AppUsuarioaplicativo'
            )
                ->findOneBy(
                    array(
                        'nrorg' => $nrOrg,
                        'nrusuarioapp' => $rows['usuario']
                    )
                );

            $codigoAssociacao = $usuarioAssocApp->getNrusuarioaplicativo();

            $this->salvaUnidadeClienteUsuarioComPapel(
                $codigoAssociacao, $rows['unidadeCliente'], $rows['papel'],
                $nrOrg, $operador, $data);

            $connection->commit();
            return true;
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function excluiUsuarioMyOrder(
        DTO\Request\Row $request, DTO\Response $response
    ){
        $codigoUsuario = $request->getRow()[0];
        $nrOrg = $this->environment->getNrOrg();

        try {
            // RETORNA NÚMERO DE APLICATIVOS CADASTRADOS
            /** @var Entities\AppUsuarioaplicativo $usuarioApp */
            $usuarioApps = $this->entityManager->getRepository(
                '\Teknisa\Model\Mymenu\Entities\AppUsuarioaplicativo'
            )
                ->findBy(
                    array(
                        'nrorg' => $nrOrg,
                        'nrusuarioapp' => $codigoUsuario
                    )
                );

            foreach ($usuarioApps as $usuarioApp){
                if ($usuarioApp->getNraplicativo() == 3)
                    $cdUsuarioApp = $usuarioApp->getNrusuarioaplicativo();
            }

            // EXCLUI PAPEIS
            $connection = $this->entityManager->getConnection();
            $query = "DELETE FROM APP_PAPELUSUCLIFIL
                       WHERE NRUSUARIOCLIFIL IN (SELECT NRUSUARIOCLIFIL
                                                   FROM APP_USUARIOCLIFIL
                                                   WHERE NRUSUARIOAPLICATIVO = :NRUSUARIOAPLICATIVO
                                                     AND NRORG = :NRORG)";
            $connection->executeQuery(
                $query, array(
                "NRUSUARIOAPLICATIVO" => $cdUsuarioApp,
                "NRORG" => $nrOrg
                )
            );

            // EXCLUI UNIDADES/CLIENTES
            $connection = $this->entityManager->getConnection();
            $query
                = "DELETE FROM APP_USUARIOCLIFIL
                    WHERE NRUSUARIOAPLICATIVO = :NRUSUARIOAPLICATIVO
                      AND NRORG = :NRORG";
            $connection->executeQuery(
                $query, array(
                "NRUSUARIOAPLICATIVO" => $cdUsuarioApp,
                "NRORG" => $nrOrg
            )
            );

            // EXCLUI ASSOCIAÇÃO AO MYORDER
            $connection = $this->entityManager->getConnection();
            $query = "DELETE FROM APP_USUARIOAPLICATIVO
                       WHERE NRUSUARIOAPP = :NRUSUARIOAPP
                         AND NRAPLICATIVO = 3
                         AND NRORG = :NRORG";
            $connection->executeQuery(
                $query,
                array("NRUSUARIOAPP" => $codigoUsuario, "NRORG" => $nrOrg)
            );

            return true;
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function excluiCliFilUsuario(
        DTO\Request\Row $request, DTO\Response $response
    ){
        $cdUsuarioCliFil = $request->getRow()[0];
        $nrOrg = $this->environment->getNrOrg();

        try {
            // EXCLUI PAPEIS
            $connection = $this->entityManager->getConnection();
            $query = "DELETE FROM APP_PAPELUSUCLIFIL
                       WHERE NRUSUARIOCLIFIL IN (SELECT NRUSUARIOCLIFIL
                                                   FROM APP_USUARIOCLIFIL
                                                   WHERE NRUSUARIOCLIFIL = :NRUSUARIOCLIFIL
                                                     AND NRORG = :NRORG)";
            $connection->executeQuery(
                $query, array(
                "NRUSUARIOCLIFIL" => $cdUsuarioCliFil,
                "NRORG" => $nrOrg
                )
            );

            // EXCLUI UNIDADES/CLIENTES
            $connection = $this->entityManager->getConnection();
            $query
                = "DELETE FROM APP_USUARIOCLIFIL
                    WHERE NRUSUARIOCLIFIL = :NRUSUARIOCLIFIL
                      AND NRORG = :NRORG";
            $connection->executeQuery(
                $query, array(
                "NRUSUARIOCLIFIL" => $cdUsuarioCliFil,
                "NRORG" => $nrOrg
                )
            );

            return true;
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}